/*
 *  This program plots a sierpinski triangle
 *   Marien van Westen (m.c.van.westen@pl.hanze.nl)
 *   march 23 2001
 */

#include "forms.h"
#include <stdlib.h>
#include <math.h>

#define MAX_RGB 3000

static FL_FORM *cl;
static FL_OBJECT *rescol, *screen, *box, *nop,  *rs, *gs, *bs;
static void create_form_cl(void);
long int no_of_points ;

typedef struct { int r, g, b; } RGBdb;

static RGBdb rgbdb[MAX_RGB];


int main(int argc, char *argv[])
{

    fl_initialize(&argc, argv, "DrawingDemo", 0, 0);

    create_form_cl();
    fl_show_form(cl, FL_PLACE_FREE, FL_FULLBORDER, "Sierpinski");
    fl_set_slider_value(rs, 128);
    fl_set_slider_value(gs, 128);
    fl_set_slider_value(bs, 128);
    fl_set_counter_value(nop,10000);
    no_of_points = 10000;
    fl_mapcolor(FL_FREE_COL4, 128, 128, 128);
    fl_redraw_object(rescol);
    
    while (fl_do_forms())
	;
    return 0;
}


static void
search_rgb(FL_OBJECT * ob, long q)
{
    int r, g, b, i;

    r = fl_get_slider_value(rs);
    g = fl_get_slider_value(gs);
    b = fl_get_slider_value(bs);

    fl_mapcolor(FL_FREE_COL4, r, g, b);
    fl_redraw_object(rescol);

}

static int freeobject_handler(FL_OBJECT * ob, int event, FL_Coord mx, FL_Coord my,
		   int key, void *xev)
{
 static int x[3], y[3];
 int n, xp, yp ;
 static int i=0 ;
 
 switch (event)
    {
    
    case FL_ENTER:
    	fl_set_cursor(FL_ObjWin(ob),XC_crosshair);
        break;  
        
    case FL_LEAVE:
        fl_reset_cursor(FL_ObjWin(ob)); 	
        break;
        
    case FL_PUSH:
         if (i==0) {x[0]=mx; y[0]= my;}
         if (i==1) {x[1]=mx; y[1]= my;}
         if (i==2) {x[2]=mx; y[2]= my;}
	 fl_line(mx,my,mx+1,my,FL_FREE_COL4);
 	 i++;
 	 if (i<3) break;	
 	 xp=x[0]; yp=y[0];
 
 	 for (i=0; i< no_of_points; i++)
     		{ 
       		n = rand() % 3;
       		xp = floor ((x[n]+xp+1)/2);
       		yp = floor ((y[n]+yp+1)/2);
       		fl_line(xp,yp,xp+1,yp,FL_FREE_COL4);
      		} 
         i=0;
   	 break;
   }
}

static void clear(FL_OBJECT * ob, long q)
{
 fl_rectf(box->x, box->y,box->w, box->h,FL_WHITE);
}

static void
nop_cb(FL_OBJECT * ob, long q)
{
 no_of_points = fl_get_counter_value(ob);   
}

static void
done_cb(FL_OBJECT * ob, long q)
{
    fl_finish();
    exit(0);
}


static void
create_form_cl(void)
{
    FL_OBJECT *obj;

    if (cl)
	return;

    cl = fl_bgn_form(FL_NO_BOX, 430, 385);
    obj = fl_add_box(FL_UP_BOX, 0, 0, 430, 385, "");
    fl_set_object_color(obj, FL_MCOL, FL_COL1);
    obj = fl_add_box(FL_DOWN_BOX, 128, 58, 284, 274, "");

    box=obj = fl_add_box(FL_FLAT_BOX, 130, 60, 280, 270, "Click on 3 spots on the plane above");
    fl_set_object_color(obj, FL_WHITE, FL_COL1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_BOTTOM);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);

    screen = obj = fl_add_free(FL_NORMAL_FREE,130, 60, 280, 270,"",freeobject_handler);

    obj = fl_add_box(FL_NO_BOX, 20, 10, 390, 30, "Sierpinski Triangle");
    fl_set_object_lcol(obj, FL_RED);
    fl_set_object_lsize(obj, FL_HUGE_SIZE);
    fl_set_object_lstyle(obj, FL_BOLD_STYLE + FL_SHADOW_STYLE);

     rs = obj = fl_add_valslider(FL_VERT_FILL_SLIDER, 20, 100, 30, 195, "");
    fl_set_object_color(obj, FL_MCOL, FL_RED);
    fl_set_slider_bounds(obj, 255, 0);
    fl_set_slider_precision(obj, 0);
    fl_set_object_callback(obj, search_rgb, 0);
    fl_set_slider_return(obj, 0);

    gs = obj = fl_add_valslider(FL_VERT_FILL_SLIDER, 50, 100, 30, 195, "");
    fl_set_object_color(obj, FL_MCOL, FL_GREEN);
    fl_set_slider_bounds(obj, 255, 0);
    fl_set_slider_precision(obj, 0);
    fl_set_object_callback(obj, search_rgb, 1);
    fl_set_slider_return(obj, 0);

    bs = obj = fl_add_valslider(FL_VERT_FILL_SLIDER, 80, 100, 30, 195, "");
    fl_set_object_color(obj, FL_MCOL, FL_BLUE);
    fl_set_slider_bounds(obj, 255, 0);
    fl_set_slider_precision(obj, 0);
    fl_set_object_callback(obj, search_rgb, 2);
    fl_set_slider_return(obj, 0);
    
    nop = obj = fl_add_counter(FL_SIMPLE_COUNTER, 20, 300, 90, 20, "Nr of Points");
    fl_set_object_color(obj, FL_MCOL, FL_COL1);
    fl_set_counter_bounds(obj, 0, 200000);
    fl_set_counter_step(obj,1000,0);
    fl_set_counter_precision(obj,0);
    fl_set_object_callback(obj, nop_cb, 0);
    
    obj = fl_add_button(FL_NORMAL_BUTTON, 25, 350, 80, 25, "Done");
    fl_set_object_callback(obj, done_cb, 0);

    obj = fl_add_button(FL_NORMAL_BUTTON, 220, 350, 80, 25, "Clear");
    fl_set_object_callback(obj, clear, 0);

    rescol = obj = fl_add_box(FL_FLAT_BOX, 20, 60, 90, 35, "");
    fl_set_object_color(obj, FL_FREE_COL4, FL_FREE_COL4);
    fl_set_object_boxtype(obj, FL_BORDER_BOX);


    fl_end_form();

}
